/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.definition;

import java.util.List;

import org.jbpm.pvm.internal.model.ExecutionImpl;
import org.jbpm.pvm.internal.model.ProcessDefinitionImpl;
import org.jbpm.pvm.internal.model.VariableDefinitionImpl;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.runtime.XpdlInstance;
import org.ow2.bonita.util.BonitaRuntimeException;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class XpdlProcess extends ProcessDefinitionImpl {

  protected ProcessDefinitionUUID processUUID;
  protected PackageDefinitionUUID packageUUID;

  protected List<ProcessParameter> parameters;

  protected XpdlProcess() { }

  public XpdlProcess(final ProcessDefinitionUUID processUUID,
      final PackageDefinitionUUID packageUUID,
      final List<ProcessParameter> parameters) {

    super();
    this.processUUID = processUUID;
    this.packageUUID = packageUUID;
    this.parameters = parameters;
  }

  @Override
  protected ExecutionImpl newProcessInstance() {
    XpdlInstance xpdlInstance = new XpdlInstance(this);
    return xpdlInstance.getRootExecution();
  }

  public ProcessDefinitionUUID getUUID() {
    return processUUID;
  }

  public void setVariableDefinitions(List<VariableDefinitionImpl> variableDefinitions) {
    this.variableDefinitions = variableDefinitions;
    hasVariableDefinitions = true; //variableDefinitions != null;
  }


  public List<ProcessParameter> getParameters() {
    return parameters;
  }

  public PackageDefinitionUUID getPackageDefinitionUUID() {
    return packageUUID;
  }

  @Override
  public String getName() {
    throw new BonitaRuntimeException("Call getName method on XPDLProcess is not allowed! We must always use ids!");
  }
}


