/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.runtime;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import org.jbpm.pvm.internal.model.ExecutionImpl;
import org.jbpm.pvm.internal.type.Variable;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.novabpm.util.Misc;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */

/**
 *  Variables in XpdlExecution are handled this way:
 *  - Process (global) variables are all in one VariableScope. This variable Scope always exists,
 *  even if it is empty.
 *  - Activity (local) variables are in a second variableScope. This variable scope only exists
 *  when there are local variables.
 *
 *  Methods getLocalVariables and getGlobalVariables are provided to have access to global and
 *  local variables of this execution.
 */
public class XpdlExecution extends ExecutionImpl {

  private static final long serialVersionUID = 1L;

  private static final Logger LOG = Logger.getLogger(XpdlExecution.class.getName());

  public static final String MAIN_INSTANCE_NAME = "mainInstance"; 
  protected XpdlInstance xpdlInstance;

  protected String iterationId;
  protected String activityInstanceId = MAIN_INSTANCE_NAME;
  protected int waitingForActivityInstanceNb;
  

  protected ActivityInstanceUUID currentActivityInstanceUUID = null;
  
  // mandatory for persistence
  protected XpdlExecution() {
    super();
  }

  @Override
  public XpdlExecution getParent() {
    return (XpdlExecution) parent;
  }
  
  @Override
  protected XpdlExecution newChildExecution() {
    XpdlExecution child = new XpdlExecution();
    child.xpdlInstance = this.getXpdlInstance();
    child.iterationId = this.getIterationId();
    return child;
  }

  @Override
  public XpdlExecution getProcessInstance() {
    return (XpdlExecution) super.getProcessInstance();
  }

  @Override
  public XpdlProcess getProcessDefinition() {
    return (XpdlProcess) processDefinition;
  }

  @Override
  public void begin() {
    XpdlInstance instance = getXpdlInstance();
    instance.setInstanceState(InstanceState.INITIAL);
    setIterationId(Misc.getUniqueId("it"));
    super.begin();
  }

  /* ITERATORS */

  public XpdlInstance getXpdlInstance() {
    return xpdlInstance;
  }

  public String getIterationId() {
    return iterationId;
  }

  public void setIterationId(String iterationId) {
    this.iterationId = iterationId;
  }

  @SuppressWarnings("unchecked")
  public Map<String, Object> getScopeVariables() {
    if (variables == null) {
      return Collections.EMPTY_MAP;
    }
    Map<String, Object> values = new HashMap<String, Object>();
    for (Map.Entry<String, Variable> entry : variables.entrySet()) {
      String name = entry.getKey();
      Variable variable = entry.getValue();
      Object value = variable.getValue();
      values.put(name, value);
    }
    return values;
  }
  
  public String getActivityInstanceId() {
    return activityInstanceId;
  }
  public void setActivityInstanceId(String activityInstanceId) {
    this.activityInstanceId = activityInstanceId;
  }
  public int getWaitingForActivityInstanceNb() {
    return waitingForActivityInstanceNb;
  }
  public void setWaitingForActivityInstanceNb(int waitingFor) {
    this.waitingForActivityInstanceNb = waitingFor;
  }
  public ActivityInstanceUUID getCurrentActivityInstanceUUID() {
    return currentActivityInstanceUUID;
  }  
  public void setCurrentActivityInstanceUUID(ActivityInstanceUUID currentActivityInstanceUUID) {
    this.currentActivityInstanceUUID = currentActivityInstanceUUID;
  }
  

}
